import subprocess
import logging

from eyes_on_me.config import config
from eyes_on_me.strategy import LightningStrategy
import PIL.Image


class WebcamStrategy(LightningStrategy):
    def _take_a_shot(self):
        logging.info("Taking a shot with webcam")
        shot_path = config["webcam-strategy"]["shot-path"]
        webcam_command = config["webcam-strategy"]["command"]
        subprocess.check_call(webcam_command % {"shot_path": shot_path},
                              shell=True)
        return shot_path

    def _get_image_lightning(self, path):
        with open(path) as fd:
            image = PIL.Image.open(fd)
            average_lightning = 128.0
            for x in range(image.size[0]):
                for y in range(image.size[1]):
                    average_lightning += max(image.getpixel((x, y)))

            average_lightning /= image.size[0] * image.size[1]
            average_lightning /= 255
            return average_lightning * config["webcam-strategy"]["coeff"]

    def get_lightning(self):
        return self._get_image_lightning(self._take_a_shot())



import subprocess
import re


kelvin_table = {
    1000: (255, 56, 0),
    1500: (255, 109, 0),
    2000: (255, 137, 18),
    2500: (255, 161, 72),
    3000: (255, 180, 107),
    3500: (255, 196, 137),
    4000: (255, 209, 163),
    4500: (255, 219, 186),
    5000: (255, 228, 206),
    5500: (255, 236, 224),
    6000: (255, 243, 239),
    6500: (255, 249, 253),
    7000: (245, 243, 255),
    7500: (235, 238, 255),
    8000: (227, 233, 255),
    8500: (220, 229, 255),
    9000: (214, 225, 255),
    9500: (208, 222, 255),
    10000: (204, 219, 255)}


def color_temperature_to_rgb(kelvin):
    normalized_kelvin = int(kelvin) / 500 * 500
    if (normalized_kelvin < min(kelvin_table.keys())):
        return kelvin_table[min(kelvin_table.keys())]
    elif (normalized_kelvin > max(kelvin_table.keys())):
        return kelvin_table[max(kelvin_table.keys())]
    else:
        return kelvin_table[normalized_kelvin]


def normalized_rgb_to_color_temperature(nrgb):
    (nr, ng, nb) = nrgb

    def rgb_deviation(x):
        nx = normalize_rgb(x[1])
        d = (abs(nx[0] - nr) +
             abs(nx[1] - ng) + abs(nx[2] - nb))
        return d

    min_kelvins = min(kelvin_table.items(),
                      key=rgb_deviation)

    return min_kelvins[0]


def normalize_rgb(rgb):
    rgb = [float(x) for x in rgb]
    median = sum(rgb) / len(rgb)
    normalized_rgb = [x / median for x in rgb]
    return normalized_rgb


def set_temperature(kalvin):
    rgb = color_temperature_to_rgb(kalvin)

    # normalize around 1.0
    normalized_rgb = normalize_rgb(rgb)
    # set via xgamma
    (r, g, b) = normalized_rgb
    subprocess.check_call("xgamma -rgamma %s" % r, shell=True)
    subprocess.check_call("xgamma -ggamma %s" % g, shell=True)
    subprocess.check_call("xgamma -bgamma %s" % b, shell=True)


def get_temperature():
    # find
    out = subprocess.check_output("xgamma 2>&1", shell=True)
    reg = re.search(
        "->.+?Red.+?([0-9]+\.[0-9]+).+?Green.+?([0-9]+\.[0-9]+),.+?Blue.+?([0-9]+\.[0-9]+)",
        out)
    (nr, ng, nb) = (float(x) for x in reg.groups())
    # find nearest values
    return normalized_rgb_to_color_temperature((nr, ng, nb))



import subprocess
import string


def set_backlight(level):
    subprocess.check_call("xbacklight -set %(level)s" % locals(), shell=True)


def get_backlight():
    return float(string.strip(subprocess.check_output("xbacklight", shell=True)))



import datetime
import astral

from eyes_on_me.config import config
from eyes_on_me.strategy import LightningStrategy


class TimedStrategy(LightningStrategy):
    def __init__(self):
        self.light_table = config['timed-strategy']['light-table']

    def _get_city_lights(self, date):
        astra = astral.Astral()
        city = astra[config['timed-strategy']['city']]
        sun = city.sun(date)
        return [(k, sun[k]) for k in
                self.light_table.keys()]

    def _lightning_for_time(self, date):
        # get lights
        lights = self._get_city_lights(date)
        # get two closer lights from table
        sorted_lights = sorted(lights, key=lambda light: light[1] - date)
        left_light = sorted_lights[1]
        right_light = sorted_lights[0]
        # approximate current level of light
        left_delta = float((date - left_light[1]).seconds)
        right_delta = float((right_light[1] - date).seconds)
        overall_delta = float((right_light[1] - left_light[1]).seconds)
        left_light_level = self.light_table[left_light[0]]
        right_light_level = self.light_table[right_light[0]]
        return (left_light_level * right_delta +
                right_light_level * left_delta) / overall_delta

    def get_lightning(self):
        astra = astral.Astral()
        city = astra[config['timed-strategy']['city']]
        return self._lightning_for_time(datetime.datetime.now(city.tz))



from eyes_on_me.config import config
import eyes_on_me.backlight
import eyes_on_me.temperature
import eyes_on_me.timed_strategy
import logging
import eyes_on_me.webcam_strategy


def choose_strategy():
    strategies = {
        'timed': eyes_on_me.timed_strategy.TimedStrategy,
        'webcam': eyes_on_me.webcam_strategy.WebcamStrategy
    }

    return strategies[config['strategy']]()


def get_backlight_for_light(light):
    backlights = [(0.0, config['backlight']['min']),
                  (0.5, config['backlight']['normal']),
                  (1.0, config['backlight']['max'])]
    # find closer values
    sorted_bl = sorted(backlights, key=lambda l: abs(light - l[0]))
    left_bl = sorted_bl[0]
    right_bl = sorted_bl[1]

    # calculate average
    left_delta = light - left_bl[0]
    right_delta = right_bl[0] - light
    overall_delta = right_bl[0] - left_bl[0]
    # TODO: zero division
    return (left_bl[1] * right_delta +
            right_bl[1] * left_delta) / overall_delta


def tune_backlight(light):
    backlight = get_backlight_for_light(light)
    logging.info("setting up backlight to %s", backlight)
    eyes_on_me.backlight.set_backlight(backlight)


def get_temperature_for_light(light):
    temperatures = [(0.0, config['wb_balance']['min']),
                    (0.5, config['wb_balance']['normal']),
                    (1.0, config['wb_balance']['max'])]
    # find closer ones
    sorted_temps = sorted(temperatures, key=lambda t: abs(light - t[0]))
    left_t = sorted_temps[0]
    right_t = sorted_temps[1]
    # calculate average
    left_delta = light - left_t[0]
    right_delta = right_t[0] - light
    overall_delta = right_t[0] - left_t[0]

    return (left_t[1] * right_delta +
            right_t[1] * left_delta) / overall_delta


def tune_wb(light):
    temperature = get_temperature_for_light(light)
    logging.info("setting up wb temperature to %s", temperature)
    eyes_on_me.temperature.set_temperature(temperature)


def tune_lights():
    strategy = choose_strategy()
    current_light = strategy.get_lightning()

    # set backlight
    tune_backlight(current_light)
    logging.info("current light is %s", current_light)
    # tune wb
    tune_wb(current_light)



from setuptools import setup


setup(
    name='eyes_on_me',
    version='0.2',
    packages=['eyes_on_me'],
    scripts=['eyes_on_me/bin/eyes_on_me'],
    license='BSD',
    long_description='wb temperature and backlight adjuster according to ',
    install_requires=["astral",
                      "PIL",
                      "daemonize"],
    include_package_data=True,
    package_data={'shared': ["etc/.eyes_on_me_rc"]},
    author="Yuriy Netesov",
    author_email="yuriy.netesov@gmail.com",
    url="https://bitbucket.org/dehun/eyes_on_me/",
)
